using System.Text;
using HabboEncoding;

namespace Tiger.Communication.Messages.Types;

public class ClientMessage
{
    private readonly byte[] _packet;
    private int _readerIndex;
    
    public int Header { get; }

    public ClientMessage(byte[] packet)
    {
        _packet = packet;
        _readerIndex = 0;
        Header = ReadB64();
    }

    public int ReadB64()
    {
        return Base64Encoding.DecodeInt32(ReadBytes(2));
    }

    public int ReadWire()
    {
        var value = VL64Encoding.DecodeInt32(GetRemainingBytes(VL64Encoding.MaxIntegerByteAmount), out var totalBytes);
        _readerIndex += totalBytes;
        return value;
    }

    public string ReadString()
    {
        var strlen = ReadB64();
        return Encoding.UTF8.GetString(ReadBytes(strlen));
    }
    
    public bool ReadBoolean()
    {
        return ReadByte() == Base64Encoding.Positive;
    }

    private byte ReadByte()
    {
        if (_packet.Length - _readerIndex < 1)
        {
            return 0;
        }

        var value = _packet[_readerIndex];
        _readerIndex++;
        return value;
    }

    private byte[] ReadBytes(int amount)
    {
        if (_packet.Length - _readerIndex < amount)
        {
            return Array.Empty<byte>();
        }

        var data = new byte[amount];
        Array.Copy(_packet, _readerIndex, data, 0, amount);
        _readerIndex += amount;
        return data;
    }
    
    private byte[] GetRemainingBytes(int max)
    {
        var data = new byte[max];
        for (var i = 0; i < max; i++)
        {
            if (_readerIndex + i >= _packet.Length)
                break;

            data[i] = _packet[_readerIndex + i];
        }

        return data;
    }

    public override string ToString()
    {
        return Encoding.UTF8.GetString(_packet);
    }
}