using System.Data;
using System.Data.Common;
using NHibernate;
using NHibernate.Engine;
using NHibernate.SqlTypes;
using NHibernate.UserTypes;

namespace Tiger.Storage.CustomTypes;

public class EnumTypeConverter<T> : IUserType where T : Enum
{
    public new bool Equals(object? x, object? y)
    {
        return x?.Equals(y) ?? y == null;
    }

    public int GetHashCode(object x)
    {
        return x.GetHashCode();
    }

    public object? NullSafeGet(DbDataReader rs, string[] names, ISessionImplementor session, object owner)
    {
        var obj = NHibernateUtil.String.NullSafeGet(rs, names[0], session);

        return obj == null ? null : Enum.Parse(typeof(T), obj.ToString() ?? string.Empty);
    }

    public void NullSafeSet(DbCommand cmd, object? value, int index, ISessionImplementor session)
    {
        if (value == null)
        {
            ((IDataParameter)cmd.Parameters[index]).Value = DBNull.Value;
        }
        else
        {
            ((IDataParameter)cmd.Parameters[index]).Value = value.ToString();
        }
    }

    public object DeepCopy(object value)
    {
        return value;
    }

    public object Replace(object original, object target, object owner)
    {
        return original;
    }

    public object Assemble(object cached, object owner)
    {
        return cached;
    }

    public object Disassemble(object value)
    {
        return value;
    }

    public SqlType[] SqlTypes => new SqlType[] { SqlTypeFactory.GetString(255) };
    public Type ReturnedType => typeof(T);
    public bool IsMutable => false;
}