using System.Data;
using System.Data.Common;
using System.Text.Json;
using NHibernate.Engine;
using NHibernate.SqlTypes;
using NHibernate.UserTypes;

namespace Tiger.Storage.CustomTypes;

public class StringListTypeConverter : IUserType
{
    public new bool Equals(object x, object y)
    {
        return object.Equals(x, y);
    }

    public int GetHashCode(object x)
    {
        return x.GetHashCode();
    }

    public object NullSafeGet(DbDataReader rs, string[] names, ISessionImplementor session, object owner)
    {
        var r = rs[names[0]];
        if (r == DBNull.Value) return null!;
        var strList = JsonSerializer.Deserialize<List<string>>(r.ToString() ?? "") ?? new List<string>();
        return strList;
    }

    public void NullSafeSet(DbCommand cmd, object value, int index, ISessionImplementor session)
    {
        if (value == null!)
        {
            ((IDataParameter)cmd.Parameters[index]).Value = DBNull.Value;
        }
        else
        {
            var strList = (List<string>)value;
            var strValue = JsonSerializer.Serialize(strList);
            ((IDataParameter)cmd.Parameters[index]).Value = strValue;
        }
    }

    public object DeepCopy(object value)
    {
        return value;
    }

    public object Replace(object original, object target, object owner)
    {
        return original;
    }

    public object Assemble(object cached, object owner)
    {
        return cached;
    }

    public object Disassemble(object value)
    {
        return value;
    }

    public SqlType[] SqlTypes => new SqlType[] { SqlTypeFactory.GetString(8000) };
    public Type ReturnedType => typeof(List<string>);
    public bool IsMutable => true;
}